from .mailing_lists import (
    EmailContactMailingListTitleConfig,
    MailingListEntryTitleConfig,
    MailingListSubscriberChangeRequestTitleConfig,
    MailingListSubscriberRequestEntryTitleConfig,
    MailingListTitleConfig,
    MailMailingListChartTitleConfig,
    UnsubscribedEmailContactMailingListTitleConfig,
)
from .mails import (
    MailEventMailTitleConfig,
    MailEventMassMailTitleConfig,
    MailStatusMassMailTitleConfig,
    MailTemplateTitleConfig,
    MailTitleConfig,
    MassMailTitleConfig,
)
